// pOx - some mac IO wrappers from Quake...

#include <Dialogs.h>
#include "mac_io.h"


/*
===============================================================================

FILE IO

===============================================================================
*/

#define MAX_HANDLES             10
FILE	*sys_handles[MAX_HANDLES];

int             findhandle (void)
{
	int             i;
	
	for (i=1 ; i<MAX_HANDLES ; i++)
		if (!sys_handles[i])
			return i;
	Sys_Error ("out of handles");
	return -1;
}

char *pc2macpath(const char *path)
{
	static char macpath[256];
	int m=0,i;
	Boolean lastSlash = false;
	
	// If "./" or ".\" appear at the start of the string (and possibly more than once), toss them
	while (path[0] == '.' && ((path[1] == '/') || (path[1] == '\\')))
	{
		path+=2;
		lastSlash = true;
	}

	// mac paths start with : if they're relative. pc paths start with / or \ if they're rooted.
	if((path[0] == '/') || (path[0] == '\\'))
	{
		path ++;
		lastSlash = true;
	}
	else
	{
		// pOx - FIXME! do this right...
		if (path[0] == '.' && path[1] == '.') {
			path+=2;
			macpath[m++]=':';
			lastSlash = false;
		}
		if (path[0] != ':')
		{
			macpath[m++]=':';
			lastSlash = false;
		}
	}
		
	// strcpy, replace / and \ with :
	for (i = 0; path[i]; i++)
	{
		if ((path[i] == '/') || (path[i] == '\\'))
		{
			// Convert / or \ to :, but only if the last character was not a slash
			// PC pathnames can take the form /blah/blah2//blah3 and be valid
			if (!lastSlash)
				macpath[m++] = ':';

			lastSlash = true;
		}
		else
		{
			macpath[m++] = path[i];
			lastSlash = false;
		}
	}

	macpath[m]='\0';
	return macpath;
}

FILE *Sys_fopen(char *path, char *perm)
{
	path=pc2macpath(path);
	return fopen(path, perm);
}

void Sys_fclose(FILE *f)
{
	fclose(f);
}

void Sys_remove (char *path)
{
	path=pc2macpath(path);
	remove (path);
}

int Sys_FileOpenRead (char *path, int *hndl)
{
	FILE    *f;
	int             i;
	
	i = findhandle ();

	f = Sys_fopen(path,"rb");
	if (!f)
	{
		*hndl = -1;
		return -1;
	}
	sys_handles[i] = f;
	*hndl = i;
	
	return filelength(f);
}

int Sys_FileOpenWrite (char *path)
{
	FILE    *f;
	int             i;
	
	i = findhandle ();

	f = Sys_fopen(path,"wb");
	if (f == NULL)
		Sys_Error ("Error opening %s: %s", path,strerror(errno));
	sys_handles[i] = f;
	
	return i;
}

void Sys_FileClose (int handle)
{
	fclose (sys_handles[handle]);
	sys_handles[handle] = NULL;
}

#undef rename
int Sys_Rename(char *oldPath,char *newPath)
{
	char oldName[256];
	strcpy(oldName,pc2macpath(oldPath));
	newPath=pc2macpath(newPath);
	return rename(oldName,newPath);
}

#if 0
FILE * Sys_GetFilePointer(int handle)
{
	if (handle < 0)
		return NULL;
	return sys_handles[handle];
}

int Sys_GetFileHandle(FILE *file)
{
	int i;
	for (i=0;i<MAX_HANDLES;i++)
		if (file == sys_handles[i])
			return i;
	return -1;
}
#endif

void Sys_FileSeek (int handle, int position)
{
	fseek (sys_handles[handle], position, SEEK_SET);
}

int Sys_FileRead (int handle, void *dest, int count)
{
	return fread (dest, 1, count, sys_handles[handle]);
}

int Sys_FileWrite (int handle, void *data, int count)
{
	return fwrite (data, 1, count, sys_handles[handle]);
}

int Sys_FileTime (char *path)
{
	FILE    *f;
	
	path=pc2macpath(path);
	f = fopen(path, "rb");
	if (f)
	{
		fclose(f);
		return 1;
	}
	
	return -1;
}

void Sys_mkdir (char *path)
{
	path=pc2macpath(path);
	mkdir(path);
}