/*  Copyright (C) 1996-1997  Id Software, Inc.

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA

    See file, 'COPYING', for details.
*/

/*
 * tyrlite/tyrlite.c
 * Modifications by Kevin Shanahan, 1999-2000
 */

#include "tyrlite.h"
// pOx - additions
#include "work.h"
#include "version.h"
#include <console.h>
#include <SIOUX.h>

#if TYRLIGHT_TEST_PHONG
#include "phong.h"
#endif

// pOx - START - Noise from Open Quartz
float noise = 0.0; // SerpLord - add 3D noise
// pOx - END - Noise from Open Quartz

float  scaledist      = 1.0;
float  scalecos       = 0.5;
float  rangescale     = 0.5;
int    worldminlight  = 0;
vec3_t minlight_color = { 255, 255, 255 }; /* defaults to white light   */
int    sunlight       = 0;
vec3_t sunlight_color = { 255, 255, 255 }; /* defaults to white light   */
vec3_t sunmangle      = { 0, 0, 16384 };   /* defaults to straight down */

byte     *filebase;
byte     *file_p;
byte     *file_end;

byte     *lit_filebase; // start of litfile data
byte     *lit_file_p;   // start of free space after litfile data
byte     *lit_file_end; // end of space for litfile data

dmodel_t *bspmodel;
int      bspfileface;  /* next surface to dispatch */

vec3_t   bsp_origin;

qboolean extrasamples;
qboolean compress_ents;
qboolean facecounter;
qboolean colored;
int		 color_bytes;// pOx - support for RGB & RGBA
qboolean litfile;
qboolean nominlimit;

qboolean worldface[MAX_MAP_FACES];// pOx - _notents support

qboolean nolightface[MAX_MAP_FACES];
vec3_t   faceoffset[MAX_MAP_FACES];

byte *GetFileSpace (int size)
{
    byte *buf;

    LOCK;
    file_p = (byte *)(((long)file_p + 3)&~3);
    buf = file_p;
    file_p += size;
    UNLOCK;
    if (file_p > file_end)
        Error ("GetFileSpace: overrun");
    return buf;
}

byte *GetLitFileSpace (int size)
{
    byte *buf;

    LOCK;
    /* align to 12 byte boundaries (match offets with 3 * GetFileSpace) */
    if ((long)lit_file_p % 12)
	lit_file_p += 12 - ((long)lit_file_p % 12);
    buf = lit_file_p;
    lit_file_p += size;
    UNLOCK;
    if (lit_file_p > lit_file_end)
        Error ("GetLitFileSpace: overrun");
    return buf;
}

// pOx - changed progress indicator to match vis
static void LightThread (void *junk)
{
    int i;
    
	printf("%i Total faces.\n", numfaces);
	printf("\n| Lighting |\n");
	
	InitWork(numfaces*2,1);
	
    while (1) {
        LOCK;
        i = bspfileface++;
        UNLOCK;
        
        if (i >= numfaces) {
        	DoneWork();
            printf("\nLighting Completed.\n\n");
            return;
        }
        LightFace (i, nolightface[i], faceoffset[i]);
        
        if (!facecounter){
        	NextWork();
        }
    }
    
    junk;// Shut up compiler warning
}

static void FindFaceOffsets( void )
{
    extern   int      nummodels;
    extern   dmodel_t dmodels[MAX_MAP_MODELS];
    int      i,j;
    entity_t *ent;
    char     name[ 20 ];
    char     *classname;
    vec3_t   org;

    memset( nolightface, 0, sizeof( nolightface ) );

    for( j = dmodels[ 0 ].firstface; j < dmodels[ 0 ].numfaces; j++ ) {
        nolightface[ j ] = 0;	
		worldface[ j ] = true;// pOx - _noents support
	}
    
    for( i = 1; i < nummodels; i++ ) {
        sprintf( name, "*%d", i );
        ent = FindEntityWithKeyPair( "model", name );
        if ( !ent )
            Error("FindFaceOffsets: Couldn't find entity for model %s.\n",
                  name );

        classname = ValueForKey ( ent, "classname" );
        if ( !strncmp( classname, "rotate_", 7 ) ) {
            int start;
            int end;

            GetVectorForKey(ent, "origin", org);

            start = dmodels[ i ].firstface;
            end = start + dmodels[ i ].numfaces;
            for( j = start; j < end; j++ ) {
                nolightface[ j ] = 300;
                faceoffset[ j ][ 0 ] = org[ 0 ];
                faceoffset[ j ][ 1 ] = org[ 1 ];
                faceoffset[ j ][ 2 ] = org[ 2 ];
            }
        }
    }
}

/*
 * =============
 *  LightWorld
 * =============
 */
static void LightWorld (void)
{
    /* align to 4 byte boundaries */
    filebase = file_p = (byte *)(((long)dlightdata + 3)&~3);

	// Use more space for bsp30 format
    if (colored && bspversion == BSP_RGBA_VERSION)
    	file_end = filebase + MAX_MAP_LIGHTING;
    else if (colored && bspversion == BSP_EQUAKE_VERSION)// pOx - RGB
    	file_end = filebase + (MAX_MAP_LIGHTING/4)*3;
	else
		file_end = filebase + (MAX_MAP_LIGHTING / 4);
	
	// Use spare space in dlightdata for lit file data
    // Need to align the start to a 12 byte boundary
    if (colored && litfile) {
		lit_filebase = file_end;
		if ((long)lit_filebase % 12)
	    	lit_filebase += 12 - ((long)lit_filebase % 12);
		lit_file_p = lit_filebase;
		lit_file_end = filebase + MAX_MAP_LIGHTING;
    }
    
    RunThreadsOn (LightThread);

    lightdatasize = file_p - filebase;

    printf ("lightdatasize: %i\n", lightdatasize);
}


/*
 * ==================
 * main
 * light modelfile
 * ==================
 */

int main (int argc, char **argv)
{
    int    i;
    double start;
    double end;
    char   source[1024];
	
	SIOUXSettings.userwindowtitle  = "\ptyrlite console log";
	SIOUXSettings.columns          = 104;
	SIOUXSettings.initializeTB     = true;
	SIOUXSettings.standalone       = true;
	SIOUXSettings.setupmenus       = true;
	SIOUXSettings.autocloseonquit  = false;
	SIOUXSettings.asktosaveonclose = false;
	SIOUXSettings.showstatusline   = false;
	
	argc = ccommand(&argv);// pOx - get the command line
	
    printf("---- TyrLite+ ----\n"
    		 VERSION " (" __DATE__ ") feature parity with PC release 0.94"	
    		 "\n(+)Includes support for OpenQuartz Noise, and BSP01 format.\n"
          	 "--------------\n"
    		 );

    for (i=1 ; i<argc ; i++) {
        if (!strcmp(argv[i],"-threads")) {
            numthreads = atoi (argv[i+1]);
            i++;
        } else if (!strcmp(argv[i],"-extra")) {
            extrasamples = true;
            printf ("extra sampling enabled\n");
        } else if (!strcmp(argv[i],"-dist")) {
            scaledist = atof (argv[i+1]);
            i++;
        } else if (!strcmp(argv[i],"-range")) {
            rangescale = atof (argv[i+1]);
            i++;
        } else if (!strcmp(argv[i],"-light")) {
            worldminlight = atof (argv[i+1]);
            i++;
        } else if (!strcmp(argv[i],"-compress")) {
            compress_ents = true;
            printf ("light entity compression enabled\n");
        } else if (!strcmp(argv[i],"-nocount")) {
            facecounter = true;
        } else if ( !strcmp(argv[i],"-colored") || !strcmp(argv[i],"-coloured") || !strcmp(argv[i],"-bsp30") ) {
            colored = true;
        	color_bytes = 4;
        	bspversion = BSP_RGBA_VERSION;
        } else if (!strcmp(argv[i],"-lit")) {
	    	litfile = true;
	    	colored = true;
        	color_bytes = 4;
        	bspversion = BSP_OLD_VERSION;
        } else if (!strcmp(argv[i],"-nominlimit")) {
            nominlimit = true;
        // pOx - START - Noise from Open Quartz
        } else if (!strcmp(argv[i],"-noise")) {
			// SerpLord - add 3D noise
			noise = atof(argv[++i]);
		// pOx - END - Noise from Open Quartz
        } else if (argv[i][0] == '-')
            Error ("Unknown option \"%s\"", argv[i]);
        else
            break;
    }

    if (i != argc - 1)
    	Error ("usage: light [-extra] [-light num] [-noise num] [-colored] [-bsp30]\n"
	       	   "             [-lit] [-nocount] [-compress] [-nominlimit] bspfile\n");

    InitThreads ();

    start = I_FloatTime ();

    strcpy (source, argv[i]);
    StripExtension (source);
    DefaultExtension (source, ".bsp");

    LoadBSPFile (source);
    LoadEntities ();

#if TYRLIGHT_TEST_PHONG
	MakeBackplanes();
	PairEdges ();
#endif

    MakeTnodes ();
    FindFaceOffsets();
	LightWorld ();
    WriteEntitiesToString ();
	WriteBSPFile (source, bspversion);
    if (litfile) WriteLitFile (source, LIT_VERSION);
	
    end = I_FloatTime ();
    printf ("%5.1f seconds elapsed\n", end-start);

#if TYRLIGHT_TEST_PHONG// DEBUG
	{
		extern int modnorms, facenorms;
		printf ("\nPHONG STATS:\n%i bent samplepoint normals.\n",modnorms);
	 	printf ("%i samplepoints used facet direction.\n",facenorms);
	}
#endif

    return 0;
}
